/**
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * beck et al. projects GmbH
 * Theresienhoehe 13
 * 80336 Munich
 * Phone: +49-89-5442530
 * mailto: info@bea.de
 */
package de.bsvrz.ars.export.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.util.Date;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import de.bsvrz.ars.export.Dataspecification;
import de.bsvrz.ars.export.DataspecificationList;
import de.bsvrz.ars.export.ExportManager;
import de.bsvrz.ars.export.ExportProfile;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hauptfenster fr den Datenexport
 * 
 * @author beck et al. projects GmbH
 * @author Phil Schrettenbrunner
 * @version $Revision: 1.1 $ / $Date: 2008/01/22 16:39:42 $ / ($Author: yvonnes $)
 * 
 */
public class ExportFrame extends JFrame implements ActionListener, FocusListener, ListSelectionListener {
    private static final long serialVersionUID = 2897949265405498461L;
    private JPanel zeitauswahlPanel, datenspezifikationPanel;
    private JTextField vonZeitpunkt, bisZeitpunkt;
    private JList datenSpezListe;
    private JButton addButton, editButton, deleteButton;
    private JButton loadButton, saveButton;
    private JButton exportButton, exitButton;
    private final JFileChooser exportFileChooser = new JFileChooser();
    private final JFileChooser datenspezFileChooser = new JFileChooser();

    private ExportProfile exportProfile;
    private DataSelectionDialog dsd;

    private static final Color errorColor = new Color(255, 204, 204);

    private final String XML_FORMAT = "XML Format (.xml)";
    private final String SQL_FORMAT = "SQL Format (.sql)";
    private final String LINE_FORMAT = "LINE Format (.txt)";


    /**
     * Erzeugt einen neuen ExportFrame (das Hauptfenster des Datenexports). Das bergebene
     * ExportProfile enthlt die Informatinen die spter fr das Durchfhren des Exports bentigt
     * werden. Mittels der GUI werden die Informationen darin noch angepasst
     * 
     * @param exportProfile Das ExportProfile, das die Informationen fr den Export enthlt. Es
     * sollte auch ein offene Verbindung zum Datenverteiler haben, da diese im GUI fr die
     * Vorauswahl bentigt wird
     * 
     */
    public ExportFrame(ExportProfile exportProfile) {
        super("Datenexport");
        this.exportProfile = exportProfile;
        this.dsd = new DataSelectionDialog(this);

        initComponents();
        listChanged();
        valueChanged(null);

        exportFileChooser.addChoosableFileFilter(new FileFilter() {
            public boolean accept(File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith(".xml");
            }

            public String getDescription() {
                return XML_FORMAT;
            }
        });
        exportFileChooser.addChoosableFileFilter(new FileFilter() {
            public boolean accept(File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith(".sql");
            }

            public String getDescription() {
                return SQL_FORMAT;
            }
        });

        if (System.getProperty("enable.lineformat") != null) {
            exportFileChooser.addChoosableFileFilter(new FileFilter() {
                public boolean accept(File f) {
                    return f.isDirectory() || f.getName().toLowerCase().endsWith(".txt");
                }

                public String getDescription() {
                    return LINE_FORMAT;
                }
            });
        }

        datenspezFileChooser.addChoosableFileFilter(new FileFilter() {
            public boolean accept(File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith(".xml");
            }

            public String getDescription() {
                return "XML Format";
            }
        });

        // Felder vorbelegen
        // wenn bis jetzt noch kein "bis" Daum gesetzt ist, setzen wir es auf
        // jetzt
        if (exportProfile.hasUntil())
            exportProfile.setUntil(new Date().getTime());
        vonZeitpunkt.setText(exportProfile.getFromAsString());
        bisZeitpunkt.setText(exportProfile.getUntilAsString());
    }


    /**
     * Fgt der Liste von Datenspezifikationen ein Element hinzu
     * 
     * @param dataspecification
     */
    public void addElement(Dataspecification dataspecification) {
        ((DataspecificationList) datenSpezListe.getModel()).add(dataspecification);
        listChanged();
        valueChanged(null);
    }


    /** 
     * Liefert die Datenspezifikation
     * @param index Index der Datenspezifikation
     * @return  die Datenspezifikation
     */
    public Dataspecification getDataspecification(int index) {
        return (Dataspecification) ((DataspecificationList) datenSpezListe.getModel()).getElementAt(index);
    }


    /**
     * ndert ein Element in der Datenspezifikation
     * 
     * @param id
     * 
     * @param dataspecification
     */
    public void editElement(int id, Dataspecification dataspecification) {
        ((DataspecificationList) datenSpezListe.getModel()).update(id, dataspecification);
        listChanged();
        valueChanged(null);
    }


    /**
     * Behandelt alle Events, die im Hauptfenster auftreten knnen: Hinzufgen, Entfernen, Laden,
     * Speichern, Exportieren und Abbrechen
     * 
     * @param ae ActionEvent
     */
    public void actionPerformed(ActionEvent ae) {
        String cmd = ae.getActionCommand();

        if (cmd == null) {
            // key stroke
            System.out.println(ae.getSource());
        } else {
            // button
            if (cmd.equals(loadButton.getActionCommand())) {
                // Datenspezifikationen aus XML-Datei lesen
                readDataspezification();
            } else if (cmd.equals(saveButton.getActionCommand())) {
                // Datenspezification in XML-Datei schreiben
                saveDataspecification();
            } else if (cmd.equals(addButton.getActionCommand())) {
                // Datenspezifikation Hinzufgen
                dsd.open();
            } else if (cmd.equals(editButton.getActionCommand())) {
                // Datenspezifikation bearbeiten
                dsd.open(datenSpezListe.getSelectedIndex());
            } else if (cmd.equals(deleteButton.getActionCommand())) {
                if (datenSpezListe.getSelectedIndex() >= 0)
                    ((DataspecificationList) datenSpezListe.getModel()).remove(datenSpezListe.getSelectedIndex());
                else
                    JOptionPane.showMessageDialog(this, "Bitte treffen Sie eine Auswahl");
                listChanged();
                valueChanged(null);
            } else if (cmd.equals(exportButton.getActionCommand())) {
                performExport();
            } else if (cmd.equals(exitButton.getActionCommand())) {
                exportProfile.getDavConnection().disconnect(false,"");
                System.exit(0);
            } else {
                // Wenn dieses else erreicht wird, ist ein
                // schwerer Fehler passiert!
                Debug.getLogger().error("ERROR - Keine Aktion fr Button hinterlegt");
            }
        }
    }


    /**
     * Fhrt das Speichern der Datenspezifikationen durch. berprft, ob alle Daten vorhanden sind,
     * dass keine Datei berschrieben wird etc
     */
    private void saveDataspecification() {
        // Speichern der Datenspezifikationen
        if (datenSpezListe.getModel().getSize() > 0) {
            int retVal = datenspezFileChooser.showDialog(this, "Datenspezifikation speichern");
            if (retVal == JFileChooser.APPROVE_OPTION) {
                try {
                    File f = datenspezFileChooser.getSelectedFile();
                    // Dateiendung checken und gegebenenfalls um .xml erweitern
                    if (!f.toString().endsWith(".xml")) {
                        f = new File(f + ".xml");
                    }
                    if (f.exists()) {
                        if (JOptionPane.showConfirmDialog(this, "Die Datei existiert bereits. berschreiben?", "berschreiben", JOptionPane.YES_NO_OPTION,
                                JOptionPane.QUESTION_MESSAGE) != JOptionPane.YES_OPTION) {
                            return; // dont overwrite
                        }
                    }
                    ((DataspecificationList) datenSpezListe.getModel()).saveToFile(f);
                } catch (IOException e) {
                    JOptionPane.showMessageDialog(this, "Die Datei kann nicht gespeichert werden:\n" + e.getMessage(),
                            "Fehler beim Speichern der Datei", JOptionPane.ERROR_MESSAGE);
                }
            }
        } else {
            JOptionPane.showMessageDialog(this, "Keine Datenspezifikationen", "Keine Datenspezifikationen", JOptionPane.WARNING_MESSAGE);
        }
    }


    /**
     * Fhrt das Einlesen der Datenspezifikationen durch
     */
    private void readDataspezification() {
        // wen schon etwas ausgewhlt ist, nachfragen, ob das ersetzt werden
        // soll
        if (datenSpezListe.getModel().getSize() > 0) {
            int ret = JOptionPane.showConfirmDialog(this,
                    "Ihre momentane Auswahl an Datenspezifikationen wird durch die Daten aus der Datei ersetzt.\nFortfahren?",
                    "Datenspezifikationen werden ersetzt", JOptionPane.WARNING_MESSAGE);
            if (ret == JOptionPane.CANCEL_OPTION)
                return;
        }

        // Dialog zeigen
        int retVal = datenspezFileChooser.showDialog(this, "Datenspezifikation laden");

        // wenn OK, dann laden
        if (retVal == JFileChooser.APPROVE_OPTION) {
            try {
                List<String> err = exportProfile.loadDataspecificationFile(datenspezFileChooser.getSelectedFile());
                if (err.size() > 0) {
                    String e = "";
                    for (String string : err)
                        e += string + "\n";
                    JOptionPane.showMessageDialog(this, "Beim Einlesen der Daten sind Fehler aufgetreten:\n" + e, "Fehler beim Lesen der Datei",
                            JOptionPane.ERROR_MESSAGE);
                }
            } catch (FileNotFoundException e) {
                JOptionPane.showMessageDialog(this, "Fehler beim ffnen der Datei:\n" + e.getMessage(), "Lesefehler", JOptionPane.ERROR_MESSAGE);
            } catch (SAXException e) {
                if (e instanceof SAXParseException) {
                    SAXParseException pe = (SAXParseException) e;
                    JOptionPane.showMessageDialog(this, "Fehler beim Einlesen der Datei. " + "Der Fehler ist in Zeile " + pe.getLineNumber()
                            + " aufgetreten.\n\nFehlermeldung:\n" + e.getMessage(), "Fehler beim Parsen der Datei", JOptionPane.ERROR_MESSAGE);
                } else {
                    JOptionPane.showMessageDialog(this, "Die Datei kann nicht verarbeitet werden:\n" + e.getMessage(), "Fehler beim Parsen der Datei",
                            JOptionPane.ERROR_MESSAGE);
                }
            } catch (IOException e) {
                JOptionPane.showMessageDialog(this, "Fehler beim ffnen der Datei:\n" + e.getMessage(), "Lesefehler", JOptionPane.ERROR_MESSAGE);
            }
        }
        listChanged();
        valueChanged(null);
    }


    /**
     * berprft, ob alle Felder ausgefllt sind und fhrt den Export durch, wenn alles OK ist.
     */
    private void performExport() {
        // Startzeit
        if (exportProfile.getFrom() < 0L) {
            JOptionPane.showMessageDialog(this, "Bitte geben Sie einen Startzeitpunkt ein.\nFormat 'TT.MM.JJJJ SS:mm:ss'", "Unvollstndige Angaben",
                    JOptionPane.WARNING_MESSAGE);
            vonZeitpunkt.grabFocus();
            return;
        }
        // Endzeit
        if (exportProfile.getUntil() == 0L) {
            JOptionPane.showMessageDialog(this, "Bitte geben Sie einen Endzeitpunkt ein.\nFormat 'TT.MM.JJJJ SS:mm:ss'", "Unvollstndige Angaben",
                    JOptionPane.WARNING_MESSAGE);
            bisZeitpunkt.grabFocus();
            return;
        }
        // End vor Start
        if (exportProfile.getFrom() > exportProfile.getUntil()) {
            JOptionPane.showMessageDialog(this, "Endzeitpunkt liegt vor dem Startzeitpunkt", "Widersprchliche Angaben", JOptionPane.WARNING_MESSAGE);
            vonZeitpunkt.grabFocus();
            return;
        }
        if (datenSpezListe.getModel().getSize() <= 0) {
            JOptionPane.showMessageDialog(this, "Sie mssen mindestens eine Datenspezifikation auswhlen", "Unvollstndige Angaben",
                    JOptionPane.WARNING_MESSAGE);
            return;
        }

        int retVal = exportFileChooser.showDialog(this, "Exportieren");
        if (retVal == JFileChooser.APPROVE_OPTION) {
            // if (exportFileChooser.getSelectedFile().canWrite()) {
            // Export kann beginnen
            if (exportFileChooser.getFileFilter().getDescription().equals(XML_FORMAT)) {
                exportProfile.setFormat(ExportProfile.Format.XML);
                if (exportFileChooser.getSelectedFile().toString().endsWith(".xml"))
                    exportProfile.setOutputFile(exportFileChooser.getSelectedFile());
                else
                    exportProfile.setOutputFile(exportFileChooser.getSelectedFile() + ".xml");
            } else if (exportFileChooser.getFileFilter().getDescription().equals(SQL_FORMAT)) {
                exportProfile.setFormat(ExportProfile.Format.SQL);
                if (exportFileChooser.getSelectedFile().toString().endsWith(".sql"))
                    exportProfile.setOutputFile(exportFileChooser.getSelectedFile());
                else
                    exportProfile.setOutputFile(exportFileChooser.getSelectedFile() + ".sql");
            } else if (exportFileChooser.getFileFilter().getDescription().equals(LINE_FORMAT)) {
                exportProfile.setFormat(ExportProfile.Format.LINE);
                if (exportFileChooser.getSelectedFile().toString().endsWith(".txt"))
                    exportProfile.setOutputFile(exportFileChooser.getSelectedFile());
                else
                    exportProfile.setOutputFile(exportFileChooser.getSelectedFile() + ".txt");
            } else {
                JOptionPane.showMessageDialog(this, "FEHLER - Kein Dateityp");
                return;
            }

            ExportManager em = new ExportManager(exportProfile, new ExportProgress(this));
            em.beginExport();
        }
    }


    /**
     * Liefert das verwendete ExportProfile zurck. Dieses enthlt alle mit der GUI gemachten
     * Einstellungen
     * 
     * @return Das ExportProfile mit allen Informationen, die ber die GUI eingegeben wurden
     */
    public ExportProfile getExportProfile() {
        return exportProfile;
    }


    /**
     * Initialisiert die GUI Komponenten und ordnet sie an entsprechender Stelle an. Die GUI wird
     * noch nicht gezeigt.
     * 
     */
    private void initComponents() {
        getContentPane().setLayout(new BorderLayout());
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // ///// Reihe Zeitauswahl
        zeitauswahlPanel = new JPanel(new GridBagLayout());
        zeitauswahlPanel.setBorder(BorderFactory.createTitledBorder("Zeitauswahl"));

        Insets i = new Insets(3, 3, 3, 3);

        // von
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.insets = i;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        zeitauswahlPanel.add(new JLabel("von:", JLabel.RIGHT), gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.insets = i;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1.0;
        vonZeitpunkt = new JTextField();
        vonZeitpunkt.setToolTipText("Anfangszeitpunkt im Format TT.MM.JJJJ SS.MM.ss");
        vonZeitpunkt.addFocusListener(this);
        zeitauswahlPanel.add(vonZeitpunkt, gbc);

        // bis
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.insets = i;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        zeitauswahlPanel.add(new JLabel("bis:", JLabel.RIGHT), gbc);

        gbc = new GridBagConstraints();
        gbc.gridx = 1;
        gbc.gridy = 1;
        gbc.insets = i;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.weightx = 1.0;
        bisZeitpunkt = new JTextField();
        bisZeitpunkt.setToolTipText("Endzeitpunkt im Format TT.MM.JJJJ SS.MM.ss");
        bisZeitpunkt.addFocusListener(this);
        zeitauswahlPanel.add(bisZeitpunkt, gbc);

        getContentPane().add(zeitauswahlPanel, BorderLayout.NORTH);

        // ///// Reihe mit Datenspezifikation
        datenspezifikationPanel = new JPanel(new BorderLayout());
        datenspezifikationPanel.setBorder(BorderFactory.createTitledBorder("Datenspezifikation"));

        // Liste
        DataspecificationList listModel = exportProfile.getDataSpecificationList();
        datenSpezListe = new JList(listModel);
        JScrollPane jsp = new JScrollPane(datenSpezListe);
        datenSpezListe.addListSelectionListener(this);
        datenSpezListe.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        datenSpezListe.addMouseListener(new MouseListener() {

            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() > 1)
                    dsd.open(datenSpezListe.getSelectedIndex());
            }

            public void mousePressed(MouseEvent e) {
                // ignore
            }
            
            public void mouseReleased(MouseEvent e) {
                // ignore
            }

            public void mouseEntered(MouseEvent e) {
                // ignore
            }

            public void mouseExited(MouseEvent e) {
                // ignore
            }

        });
        JPanel jp = new JPanel(new BorderLayout());
        jp.add(jsp, BorderLayout.CENTER);
        jp.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        datenspezifikationPanel.add(jp, BorderLayout.CENTER);

        // Buttons
        addButton = new JButton("Hinzufgen");
        addButton.addActionListener(this);
        deleteButton = new JButton("Entfernen");
        deleteButton.addActionListener(this);
        editButton = new JButton("Bearbeiten");
        editButton.addActionListener(this);
        loadButton = new JButton("Laden");
        loadButton.addActionListener(this);
        saveButton = new JButton("Speichern");
        saveButton.addActionListener(this);
        JPanel ButtonPanel = new JPanel(new GridBagLayout());
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(5, 0, 5, 5);
        ButtonPanel.add(addButton, gbc);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0, 0, 5, 5);
        ButtonPanel.add(editButton, gbc);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0, 0, 5, 5);
        ButtonPanel.add(deleteButton, gbc);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0, 0, 20, 5);
        ButtonPanel.add(new JLabel(""), gbc);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0, 0, 5, 5);
        ButtonPanel.add(loadButton, gbc);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 5;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0, 0, 5, 5);
        ButtonPanel.add(saveButton, gbc);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 6;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0, 0, 5, 5);
        ButtonPanel.add(new JLabel(""), gbc);
        datenspezifikationPanel.add(ButtonPanel, BorderLayout.EAST);

        getContentPane().add(datenspezifikationPanel, BorderLayout.CENTER);

        // ///// Button Reihe
        ButtonPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING));
        exportButton = new JButton("Exportieren");
        exportButton.addActionListener(this);
        ButtonPanel.add(exportButton);

        getRootPane().setDefaultButton(exportButton);

        exitButton = new JButton("Beenden");
        exitButton.addActionListener(this);
        ButtonPanel.add(exitButton);

        getContentPane().add(ButtonPanel, BorderLayout.SOUTH);
    }


    /**
     * Aktiviert/Deaktiviert den Save Button
     */
    private void listChanged() {
        if (datenSpezListe.getModel().getSize() > 0)
            saveButton.setEnabled(true);
        else
            saveButton.setEnabled(false);
    }


    public void focusGained(FocusEvent e) { // unimportant
    }


    public void focusLost(FocusEvent e) {
        JTextField t;
        if (e.getSource() == vonZeitpunkt) {
            // VonZeitpunkt
            t = (JTextField) e.getSource();

            try {
                exportProfile.setFrom(t.getText());
                t.setText(exportProfile.getFromAsString());
                t.setBackground(Color.WHITE);
            } catch (ParseException e1) {
                t.setBackground(errorColor);
            }

        } else if (e.getSource() == bisZeitpunkt) {
            // BisZeitpunkt
            t = (JTextField) e.getSource();

            try {
                exportProfile.setUntil(t.getText());
                t.setText(exportProfile.getUntilAsString());
                t.setBackground(Color.WHITE);
            } catch (ParseException e1) {
                t.setBackground(errorColor);
            }
        }
    }


    public void valueChanged(ListSelectionEvent e) {
        if (datenSpezListe.getSelectedIndex() < 0) {
            editButton.setEnabled(false);
            deleteButton.setEnabled(false);
        } else {
            editButton.setEnabled(true);
            deleteButton.setEnabled(true);
        }
    }
}
